@@ -2,20 +2,16 @@ import re
 import binascii
 import email
 from email import quoprimime, base64mime
+from email import charset as _charset
+
+if str == bytes:
+    message_from_bytes = email.message_from_string
+    message_from_binary_file = email.message_from_file
+else:
+    message_from_bytes = email.message_from_bytes
+    message_from_binary_file = email.message_from_binary_file
 
 ## please import this file if you are using the email module
-#
-# a "monkey patch" to unify the behaviour of python 2.5 2.6 2.7
-# when generating header files, see http://bugs.python.org/issue1974
-# and https://hg.python.org/cpython/rev/5deb27042e5a/
-# can go away once the minimum requirement is python 2.7
-_oldheaderinit = email.Header.Header.__init__
-def _unifiedheaderinit(self, *args, **kw):
-    # override continuation_ws
-    kw['continuation_ws'] = ' '
-    _oldheaderinit(self, *args, **kw)
-email.Header.Header.__dict__['__init__'] = _unifiedheaderinit
-##
 
 # Match encoded-word strings in the form =?charset?q?Hello_World?=
 ecre = re.compile(r'''
@@ -51,7 +47,7 @@ def decode_header(header):
     # If it is a Header object, we can just return the encoded chunks.
     if hasattr(header, '_chunks'):
         return [(_charset._encode(string, str(charset)), str(charset))
-                    for string, charset in header._chunks]
+                for string, charset in header._chunks]
     # If no encoding, just return the header with no charset.
     if not ecre.search(header):
         return [(header, None)]
@@ -76,10 +72,9 @@ def decode_header(header):
                 words.append((encoded, encoding, charset))
     # Now loop over words and remove words that consist of whitespace
     # between two encoded strings.
-    import sys
     droplist = []
     for n, w in enumerate(words):
-        if n>1 and w[1] and words[n-2][1] and words[n-1][0].isspace():
+        if n > 1 and w[1] and words[n-2][1] and words[n-1][0].isspace():
             droplist.append(n-1)
     for d in reversed(droplist):
         del words[d]
@@ -96,7 +91,8 @@ def decode_header(header):
             word = quoprimime.header_decode(encoded_string)
             decoded_words.append((word, charset))
         elif encoding == 'b':
-            paderr = len(encoded_string) % 4   # Postel's law: add missing padding
+            # Postel's law: add missing padding
+            paderr = len(encoded_string) % 4
             if paderr:
                 encoded_string += '==='[:4 - paderr]
             try:
@@ -112,8 +108,8 @@ def decode_header(header):
     collapsed = []
     last_word = last_charset = None
     for word, charset in decoded_words:
-        if isinstance(word, str):
-            pass
+        if isinstance(word, str) and bytes != str:
+            word = bytes(word, 'raw-unicode-escape')
         if last_word is None:
             last_word = word
             last_charset = charset
@@ -128,4 +124,3 @@ def decode_header(header):
             last_word += word
     collapsed.append((last_word, last_charset))
     return collapsed
-
