@@ -27,7 +27,7 @@ import sys
 # Otherwise the error is reraised.
 _modules = {
     'mysql': ('MySQLdb',),
-    'postgresql': ('psycopg',),
+    'postgresql': ('psycopg2',),
     'sqlite': ('pysqlite', 'pysqlite2', 'sqlite3', '_sqlite3', 'sqlite'),
 }
 
@@ -39,7 +39,7 @@ def get_backend(name):
         return vars[name]
     # import the backend module
     module_name = 'back_%s' % name
-    module = __import__(module_name, vars)
+    module = __import__(module_name, vars, level=1)
     vars[name] = module
     return module
 
@@ -49,9 +49,16 @@ def have_backend(name):
         get_backend(name)
         return 1
     except ImportError as e:
-        for name in _modules.get(name, (name,)):
-            if str(e).startswith('No module named %s'%name):
-                return 0
+        if hasattr(e, 'name'):
+            modname = e.name
+        else:
+            modname = e.args[0][16:] if e.args[0].startswith('No module named ') else None
+
+        # It's always ok if memorydb is not found
+        if modname.endswith('back_memorydb'):
+            return 0
+        if modname and (modname in _modules.get(name, (name,))):
+            return 0
         raise
     return 0
 
@@ -61,9 +68,15 @@ def list_backends():
     This function has side-effect of registering backward-compatible
     globals for all available backends.
 
+    Note: Since memorydb does not live in the backends directory, it will
+    never be found in the default setup. It *can* be enabled by preloading
+    test/memorydb and injecting into roundup.backends. So the normal user
+    can never configure memorydb but it makes using the tests easier
+    because we do not need to monkey-patch list_backends.
+
     '''
     l = []
-    for name in 'anydbm', 'mysql', 'sqlite', 'postgresql':
+    for name in 'anydbm', 'mysql', 'sqlite', 'postgresql', 'memorydb':
         if have_backend(name):
             l.append(name)
     return l
