@@ -15,27 +15,28 @@
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 #
-#$Id: blobfiles.py,v 1.24 2008/02/07 00:57:59 richard Exp $
-'''This module exports file storage for roundup backends.
+"""This module exports file storage for roundup backends.
 Files are stored into a directory hierarchy.
-'''
+"""
 __docformat__ = 'restructuredtext'
 
 import os
 
+
 def files_in_dir(dir):
     if not os.path.exists(dir):
         return 0
     num_files = 0
     for dir_entry in os.listdir(dir):
-        full_filename = os.path.join(dir,dir_entry)
+        full_filename = os.path.join(dir, dir_entry)
         if os.path.isfile(full_filename):
             num_files = num_files + 1
         elif os.path.isdir(full_filename):
             num_files = num_files + files_in_dir(full_filename)
     return num_files
 
-class FileStorage:
+
+class FileStorage(object):
     """Store files in some directory structure
 
     Some databases do not permit the storage of arbitrary data (i.e.,
@@ -64,7 +65,7 @@ class FileStorage:
 
     Edit Procotol
     -------------
-    
+
     When a file is created or edited, the following protocol is used:
 
     1. The new content of the file is placed in 'file.tmp'.
@@ -92,14 +93,14 @@ class FileStorage:
     or two web clients running with a multi-process server) attempt
     edits at the same time, both will write to 'file.tmp', and the
     results will be indeterminate.
-    
+
     Crash Analysis
     --------------
-    
+
     There are several situations that may occur if a crash (whether
     because the machine crashes, because an unhandled Python exception
     is raised, or because the Python process is killed) occurs.
-    
+
     Complexity ensues because backuping up an RDBMS is generally more
     complex than simply copying a file.  Instead, some command is run
     which stores a snapshot of the database in a file.  So, if you
@@ -204,25 +205,25 @@ class FileStorage:
     database will be consistent, so long as unreferenced 'file.v'
     files are never removed until after the database has been backed
     up.
-    """    
+    """
 
     tempext = '.tmp'
     """The suffix added to files indicating that they are uncommitted."""
-    
+
     def __init__(self, umask):
         self.umask = umask
 
     def subdirFilename(self, classname, nodeid, property=None):
         """Determine what the filename and subdir for nodeid + classname is."""
         if property:
-            name = '%s%s.%s'%(classname, nodeid, property)
+            name = '%s%s.%s' % (classname, nodeid, property)
         else:
             # roundupdb.FileClass never specified the property name, so don't
             # include it
-            name = '%s%s'%(classname, nodeid)
+            name = '%s%s' % (classname, nodeid)
 
         # have a separate subdir for every thousand messages
-        subdir = str(int(nodeid) / 1000)
+        subdir = str(int(nodeid) // 1000)
         return os.path.join(subdir, name)
 
     def _tempfile(self, filename):
@@ -232,16 +233,29 @@ class FileStorage:
 
         return filename + self.tempext
 
+    def _editInProgress(self, classname, nodeid, property):
+        """Return true if the file indicated is being edited.
+
+        returns -- True if the current transaction includes an edit to
+        the file indicated."""
+
+        for method, args in self.transactions:
+            if (method == self.doStoreFile and
+                    args == (classname, nodeid, property)):
+                return True
+
+        return False
+
     def filename(self, classname, nodeid, property=None, create=0):
-        '''Determine what the filename for the given node and optionally
+        """Determine what the filename for the given node and optionally
         property is.
 
         Try a variety of different filenames - the file could be in the
         usual place, or it could be in a temp file pre-commit *or* it
         could be in an old-style, backwards-compatible flat directory.
-        '''
-        filename  = os.path.join(self.dir, 'files', classname,
-                                 self.subdirFilename(classname, nodeid, property))
+        """
+        filename = os.path.join(self.dir, 'files', classname,
+                                self.subdirFilename(classname, nodeid, property))
         # If the caller is going to create the file, return the
         # post-commit filename.  It is the callers responsibility to
         # add self.tempext when actually creating the file.
@@ -252,18 +266,15 @@ class FileStorage:
 
         # If an edit to this file is in progress, then return the name
         # of the temporary file containing the edited content.
-        for method, args in self.transactions:
-            if (method == self.doStoreFile and
-                    args == (classname, nodeid, property)):
-                # There is an edit in progress for this file.
-                if not os.path.exists(tempfile):
-                    raise IOError('content file for %s not found'%tempfile)
-                return tempfile
+        if self._editInProgress(classname, nodeid, property):
+            if not os.path.exists(tempfile):
+                raise IOError('content file for %s not found' % tempfile)
+            return tempfile
 
         if os.path.exists(filename):
             return filename
 
-        # Otherwise, if the temporary file exists, then the probable 
+        # Otherwise, if the temporary file exists, then the probable
         # explanation is that a crash occurred between the point that
         # the database entry recording the creation of the file
         # occured and the point at which the file was renamed from the
@@ -272,33 +283,37 @@ class FileStorage:
             try:
                 # Clean up, by performing the commit now.
                 os.rename(tempfile, filename)
-            except:
+            except OSError:
                 pass
             # If two Roundup clients both try to rename the file
             # at the same time, only one of them will succeed.
             # So, tolerate such an error -- but no other.
             if not os.path.exists(filename):
-                raise IOError('content file for %s not found'%filename)
+                raise IOError('content file for %s not found' % filename)
             return filename
 
         # ok, try flat (very old-style)
         if property:
-            filename = os.path.join(self.dir, 'files', '%s%s.%s'%(classname,
-                nodeid, property))
+            filename = os.path.join(self.dir, 'files', '%s%s.%s' % (
+                classname, nodeid, property))
         else:
-            filename = os.path.join(self.dir, 'files', '%s%s'%(classname,
-                nodeid))
+            filename = os.path.join(self.dir, 'files', '%s%s' % (classname,
+                                                                 nodeid))
         if os.path.exists(filename):
             return filename
 
         # file just ain't there
-        raise IOError('content file for %s not found'%filename)
+        raise IOError('content file for %s not found' % filename)
+
+    def filesize(self, classname, nodeid, property=None, create=0):
+        filename = self.filename(classname, nodeid, property, create)
+        return os.path.getsize(filename)
 
     def storefile(self, classname, nodeid, property, content):
-        '''Store the content of the file in the database. The property may be
+        """Store the content of the file in the database. The property may be
            None, in which case the filename does not indicate which property
            is being saved.
-        '''
+        """
         # determine the name of the file to write to
         name = self.filename(classname, nodeid, property, create=1)
 
@@ -310,19 +325,21 @@ class FileStorage:
         name = self._tempfile(name)
 
         # make sure we don't register the rename action more than once
-        if not os.path.exists(name):
+        if not self._editInProgress(classname, nodeid, property):
             # save off the rename action
             self.transactions.append((self.doStoreFile, (classname, nodeid,
-                property)))
+                                                         property)))
         # always set umask before writing to make sure we have the proper one
         # in multi-tracker (i.e. multi-umask) or modpython scenarios
         # the umask may have changed since last we set it.
         os.umask(self.umask)
-        open(name, 'wb').write(content)
+        fd = open(name, 'wb')
+        fd.write(content)
+        fd.close()
 
     def getfile(self, classname, nodeid, property):
-        '''Get the content of the file in the database.
-        '''
+        """Get the content of the file in the database.
+        """
         filename = self.filename(classname, nodeid, property)
 
         f = open(filename, 'rb')
@@ -333,14 +350,14 @@ class FileStorage:
             f.close()
 
     def numfiles(self):
-        '''Get number of files in storage, even across subdirectories.
-        '''
+        """Get number of files in storage, even across subdirectories.
+        """
         files_dir = os.path.join(self.dir, 'files')
         return files_in_dir(files_dir)
 
     def doStoreFile(self, classname, nodeid, property, **databases):
-        '''Store the file as part of a transaction commit.
-        '''
+        """Store the file as part of a transaction commit.
+        """
         # determine the name of the file to write to
         name = self.filename(classname, nodeid, property, 1)
 
@@ -364,8 +381,8 @@ class FileStorage:
         return (classname, nodeid)
 
     def rollbackStoreFile(self, classname, nodeid, property, **databases):
-        '''Remove the temp file as a part of a rollback
-        '''
+        """Remove the temp file as a part of a rollback
+        """
         # determine the name of the file to delete
         name = self.filename(classname, nodeid, property)
         if not name.endswith(self.tempext):
@@ -373,19 +390,19 @@ class FileStorage:
         os.remove(name)
 
     def isStoreFile(self, classname, nodeid):
-        '''See if there is actually any FileStorage for this node.
+        """See if there is actually any FileStorage for this node.
            Is there a better way than using self.filename?
-        '''
+        """
         try:
-            fname = self.filename(classname, nodeid)
+            fname = self.filename(classname, nodeid)  # noqa: F841
             return True
         except IOError:
             return False
 
     def destroy(self, classname, nodeid):
-        '''If there is actually FileStorage for this node
+        """If there is actually FileStorage for this node
            remove it from the filesystem
-        '''
+        """
         if self.isStoreFile(classname, nodeid):
             os.remove(self.filename(classname, nodeid))
 
