@@ -16,48 +16,47 @@
 from roundup import i18n
 from roundup.cgi.PageTemplates import Expressions, PathIterator, TALES
 from roundup.cgi.TAL import TALInterpreter
+from roundup.anypy.strings import us2u, u2s
 
 ### Translation classes
 
+
 class TranslationServiceMixin:
 
-    OUTPUT_ENCODING = "utf-8"
-
     def translate(self, domain, msgid, mapping=None,
-        context=None, target_language=None, default=None
-    ):
+                  context=None, target_language=None, default=None):
         _msg = self.gettext(msgid)
-        #print ("TRANSLATE", msgid, _msg, mapping, context)
+        # print ("TRANSLATE", msgid, _msg, mapping, context)
         _msg = TALInterpreter.interpolate(_msg, mapping)
         return _msg
 
-    def gettext(self, msgid):
-        if not isinstance(msgid, unicode):
-            msgid = unicode(msgid, 'utf8')
-        msgtrans=self.ugettext(msgid)
-        return msgtrans.encode(self.OUTPUT_ENCODING)
+    if hasattr(i18n.RoundupTranslations, 'ugettext'):
+        def gettext(self, msgid):
+            msgid = us2u(msgid)
+            msgtrans = self.ugettext(msgid)
+            return u2s(msgtrans)
+
+        def ngettext(self, singular, plural, number):
+            singular = us2u(singular)
+            plural = us2u(plural)
+            msgtrans = self.ungettext(singular, plural, number)
+            return u2s(msgtrans)
 
-    def ngettext(self, singular, plural, number):
-        if not isinstance(singular, unicode):
-            singular = unicode(singular, 'utf8')
-        if not isinstance(plural, unicode):
-            plural = unicode(plural, 'utf8')
-        msgtrans=self.ungettext(singular, plural, number)
-        return msgtrans.encode(self.OUTPUT_ENCODING)
 
 class TranslationService(TranslationServiceMixin, i18n.RoundupTranslations):
     pass
 
+
 class NullTranslationService(TranslationServiceMixin,
-        i18n.RoundupNullTranslations):
-    def ugettext(self, message):
-        if self._fallback:
-            return self._fallback.ugettext(message)
-        # Sometimes the untranslatable message is a UTF-8 encoded string
-        # (thanks to PageTemplate's internals).
-        if not isinstance(message, unicode):
-            return unicode(message, 'utf8')
-        return message
+                             i18n.RoundupNullTranslations):
+    if hasattr(i18n.RoundupNullTranslations, 'ugettext'):
+        def ugettext(self, message):
+            if self._fallback:
+                return self._fallback.ugettext(message)
+            # Sometimes the untranslatable message is a UTF-8 encoded string
+            # (thanks to PageTemplate's internals).
+            message = us2u(message)
+            return message
 
 ### TAL patching
 #
@@ -68,6 +67,7 @@ class NullTranslationService(TranslationServiceMixin,
 # client interface.
 #
 
+
 class Context(TALES.Context):
 
     def __init__(self, compiler, contexts):
@@ -83,8 +83,10 @@ class Context(TALES.Context):
         if context is None:
             context = self.contexts.get('here')
         return self.i18n.translate(domain, msgid,
-            mapping=mapping, context=context, default=default,
-            target_language=target_language)
+                                   mapping=mapping, context=context,
+                                   default=default,
+                                   target_language=target_language)
+
 
 class Engine(TALES.Engine):
 
@@ -96,6 +98,7 @@ class Engine(TALES.Engine):
                 kwcontexts = contexts
         return Context(self, kwcontexts)
 
+
 # patching TAL like this is a dirty hack,
 # but i see no other way to specify different Context class
 Expressions._engine = Engine(PathIterator.Iterator)
@@ -103,10 +106,10 @@ Expressions.installHandlers(Expressions._engine)
 
 ### main API function
 
+
 def get_translation(language=None, tracker_home=None,
-    translation_class=TranslationService,
-    null_translation_class=NullTranslationService
-):
+                    translation_class=TranslationService,
+                    null_translation_class=NullTranslationService):
     """Return Translation object for given language and domain
 
     Arguments 'translation_class' and 'null_translation_class'
@@ -114,8 +117,8 @@ def get_translation(language=None, tracker_home=None,
     and non-existing translations, respectively.
     """
     return i18n.get_translation(language=language,
-        tracker_home=tracker_home,
-        translation_class=translation_class,
-        null_translation_class=null_translation_class)
+                                tracker_home=tracker_home,
+                                translation_class=translation_class,
+                                null_translation_class=null_translation_class)
 
 # vim: set et sts=4 sw=4 :
