@@ -38,7 +38,7 @@ class Iterator:
         try:
             inner = getattr(self._inner, 'it_' + name)
         except AttributeError:
-            raise AttributeError, name
+            raise AttributeError(name)
         return inner(self)
 
     def next(self):
@@ -84,7 +84,7 @@ class Iterator:
             s = s + r * rct
         return s
 
-    def roman(self, lower=string.lower):
+    def roman(self, lower=lambda x:x.lower):
         return lower(self.Roman())
 
     def first(self, name=None):
@@ -169,7 +169,7 @@ class IterInner(InnerBase):
 
     def prep_next(self, it):
         try:
-            it._next = it.seq.next()
+            it._next = next(it.seq)
         except StopIteration:
             it._prep_next = self.no_next
             it.end = 1
@@ -181,7 +181,7 @@ class IterIter:
     def __init__(self, it):
         self.it = it
         self.skip = it.nextIndex > 0 and not it.end
-    def next(self):
+    def __next__(self):
         it = self.it
         if self.skip:
             self.skip = 0
@@ -189,6 +189,8 @@ class IterIter:
         if it.next():
             return it.item
         raise StopIteration
+    # Python 2 compatibility:
+    next = __next__
 
 seqInner = SeqInner()
 iterInner = IterInner()
