@@ -2,27 +2,22 @@
 
 __docformat__ = 'restructuredtext'
 
-import os.path
-from chameleon import PageTemplateLoader
+import chameleon
 
-from roundup.cgi.templating import StringIO, context, find_template, TemplatesBase
+from roundup.cgi.templating import context, TALLoaderBase
+from roundup.anypy.strings import s2u
 
-class Templates(TemplatesBase):
+
+class Loader(TALLoaderBase):
     def __init__(self, dir):
         self.dir = dir
-        self.loader = PageTemplateLoader(dir)
+        self.loader = chameleon.PageTemplateLoader(dir)
 
-    def get(self, name, extension=None):
-        # default the name to "home"
-        if name is None:
-            name = 'home'
-        elif extension is None and '.' in name:
-            # split name
-            name, extension = name.split('.')
-
-        src, filename = find_template(self.dir, name, extension)
+    def load(self, tplname):
+        src, filename = self._find(tplname)
         return RoundupPageTemplate(self.loader.load(src))
 
+
 class RoundupPageTemplate(object):
     def __init__(self, pt):
         self._pt = pt
@@ -33,8 +28,9 @@ class RoundupPageTemplate(object):
 
         def translate(msgid, domain=None, mapping=None, default=None):
             result = client.translator.translate(domain, msgid,
-                         mapping=mapping, default=default)
-            return unicode(result, client.translator.OUTPUT_ENCODING)
+                                                 mapping=mapping,
+                                                 default=default)
+            return s2u(result)
 
         output = self._pt.render(None, translate, **c)
         return output.encode(client.charset)
@@ -44,4 +40,3 @@ class RoundupPageTemplate(object):
 
     def __getattr__(self, name):
         return getattr(self._pt, name)
-
