@@ -8,47 +8,30 @@ import mimetypes
 import os
 import os.path
 
-from roundup.cgi.templating import StringIO, context, translationService, find_template, TemplatesBase
-from roundup.cgi.PageTemplates import PageTemplate, GlobalTranslationService
+from roundup.cgi.templating import StringIO, context, TALLoaderBase
+from roundup.cgi.PageTemplates import PageTemplate
 from roundup.cgi.PageTemplates.Expressions import getEngine
 from roundup.cgi.TAL import TALInterpreter
 
-GlobalTranslationService.setGlobalTranslationService(translationService)
 
-class Templates(TemplatesBase):
+class Loader(TALLoaderBase):
     templates = {}
 
     def __init__(self, dir):
         self.dir = dir
 
-    def get(self, name, extension=None):
-        """ Interface to get a template, possibly loading a compiled template.
-
-            "name" and "extension" indicate the template we're after, which in
-            most cases will be "name.extension". If "extension" is None, then
-            we look for a template just called "name" with no extension.
-
-            If the file "name.extension" doesn't exist, we look for
-            "_generic.extension" as a fallback.
-        """
-        # default the name to "home"
-        if name is None:
-            name = 'home'
-        elif extension is None and '.' in name:
-            # split name
-            name, extension = name.split('.')
-
+    def load(self, tplname):
         # find the source
-        src, filename = find_template(self.dir, name, extension)
+        src, filename = self._find(tplname)
 
         # has it changed?
         try:
             stime = os.stat(src)[os.path.stat.ST_MTIME]
-        except os.error, error:
+        except os.error as error:
             if error.errno != errno.ENOENT:
                 raise
 
-        if self.templates.has_key(src) and \
+        if src in self.templates and \
                 stime <= self.templates[src].mtime:
             # compiled template is up to date
             return self.templates[src]
@@ -57,7 +40,8 @@ class Templates(TemplatesBase):
         pt = RoundupPageTemplate()
         # use pt_edit so we can pass the content_type guess too
         content_type = mimetypes.guess_type(filename)[0] or 'text/html'
-        pt.pt_edit(open(src).read(), content_type)
+        with open(src) as srcd:
+            pt.pt_edit(srcd.read(), content_type)
         pt.id = filename
         pt.mtime = stime
         # Add it to the cache.  We cannot do this until the template
@@ -78,6 +62,7 @@ class Templates(TemplatesBase):
         self.templates[src] = pt
         return pt
 
+
 class RoundupPageTemplate(PageTemplate.PageTemplate):
     """A Roundup-specific PageTemplate.
 
@@ -95,16 +80,16 @@ class RoundupPageTemplate(PageTemplate.PageTemplate):
         __traceback_supplement__ = (PageTemplate.PageTemplateTracebackSupplement, self)
 
         if self._v_errors:
-            raise PageTemplate.PTRuntimeError, \
-                'Page Template %s has errors.'%self.id
+            raise PageTemplate.PTRuntimeError('Page Template %s has errors.' %
+                                              self.id)
 
         # figure the context
         c = context(client, self, classname, request)
         c.update({'options': options})
 
         # and go
-        output = StringIO.StringIO()
+        output = StringIO()
         TALInterpreter.TALInterpreter(self._v_program, self.macros,
-            getEngine().getContext(c), output, tal=1, strictinsert=0)()
+                                      getEngine().getContext(c), output,
+                                      tal=1, strictinsert=0)()
         return output.getvalue()
-
