@@ -3,28 +3,46 @@
 
 __docformat__ = 'restructuredtext'
 
-from roundup.exceptions import LoginError, Unauthorised
-import cgi
+from roundup.exceptions import LoginError, Unauthorised  # noqa: F401
 
-class HTTPException(BaseException):
+from roundup.anypy.html import html_escape
+
+from roundup.exceptions import RoundupException
+
+
+class RoundupCGIException(RoundupException):
     pass
 
+
+class HTTPException(RoundupCGIException):
+    pass
+
+
 class Redirect(HTTPException):
     pass
 
+
 class NotFound(HTTPException):
     pass
 
+
 class NotModified(HTTPException):
     pass
 
-class DetectorError(BaseException):
+
+class PreconditionFailed(HTTPException):
+    pass
+
+
+class DetectorError(RoundupException):
     """Raised when a detector throws an exception.
 Contains details of the exception."""
     def __init__(self, subject, html, txt):
         self.subject = subject
         self.html = html
         self.txt = txt
+        BaseException.__init__(self, subject + ' ' + txt)
+
 
 class FormError(ValueError):
     """An 'expected' exception occurred during form parsing.
@@ -38,13 +56,23 @@ class FormError(ValueError):
     """
     pass
 
-class SendFile(BaseException):
+
+class IndexerQueryError(RoundupException):
+    """Raised to handle errors from FTS searches due to query
+       syntax errors.
+    """
+    pass
+
+
+class SendFile(RoundupException):
     """Send a file from the database."""
 
-class SendStaticFile(BaseException):
+
+class SendStaticFile(RoundupException):
     """Send a static file from the instance html directory."""
 
-class SeriousError(BaseException):
+
+class SeriousError(RoundupException):
     """Raised when we can't reasonably display an error message on a
     templated page.
 
@@ -59,6 +87,6 @@ class SeriousError(BaseException):
 <body class="body" marginwidth="0" marginheight="0">
  <p class="error-message">%s</p>
 </body></html>
-"""%cgi.escape(self.args[0])
+""" % html_escape(self.args[0])
 
 # vim: set filetype=python sts=4 sw=4 et si :
