@@ -2,35 +2,37 @@
 """
 __docformat__ = 'restructuredtext'
 
-import time, quopri, os, socket, smtplib, re, sys, traceback, email
+import logging
+import os
+import smtplib
+import socket
+import sys
+import time
+import traceback
 
-from cStringIO import StringIO
+from email import charset
+from email.charset import Charset
+from email.header import Header
+from email.mime.base import MIMEBase
+from email.mime.multipart import MIMEMultipart
+from email.mime.nonmultipart import MIMENonMultipart
+from email.mime.text import MIMEText
+from email.utils import formatdate, specialsre, escapesre
 
+from roundup.anypy import email_                # noqa: F401 defines functions
+from roundup.anypy.strings import b2s, s2u
 from roundup import __version__
 from roundup.date import get_timezone, Date
 
-from email.Utils import formatdate, formataddr, specialsre, escapesre
-from email.Message import Message
-from email.Header import Header
-from email.MIMEBase import MIMEBase
-from email.MIMEText import MIMEText
-from email.MIMEMultipart import MIMEMultipart
-
 try:
-    import pyme, pyme.core
+    import gpg, gpg.core                        # noqa: E401
 except ImportError:
-    pyme = None
+    gpg = None
 
 
 class MessageSendError(RuntimeError):
     pass
 
-def encode_quopri(msg):
-    orig = msg.get_payload()
-    encdata = quopri.encodestring(orig)
-    msg.set_payload(encdata)
-    del msg['Content-Transfer-Encoding']
-    msg['Content-Transfer-Encoding'] = 'quoted-printable'
 
 def nice_sender_header(name, address, charset):
     # construct an address header so it's as human-readable as possible
@@ -38,23 +40,25 @@ def nice_sender_header(name, address, charset):
     if not name:
         return address
     try:
-        encname = name.encode('ASCII')
+        encname = b2s(name.encode('ASCII'))
     except UnicodeEncodeError:
         # use Header to encode correctly.
         encname = Header(name, charset=charset).encode()
 
     # the important bits of formataddr()
     if specialsre.search(encname):
-        encname = '"%s"'%escapesre.sub(r'\\\g<0>', encname)
+        encname = '"%s"' % escapesre.sub(r'\\\g<0>', encname)
 
     # now format the header as a string - don't return a Header as anonymous
     # headers play poorly with Messages (eg. won't get wrapped properly)
-    return '%s <%s>'%(encname, address)
+    return '%s <%s>' % (encname, address)
+
 
 class Mailer:
     """Roundup-specific mail sending."""
     def __init__(self, config):
         self.config = config
+        self.logger = logging.getLogger('roundup.mailer')
 
         # set to indicate to roundup not to actually _send_ email
         # this var must contain a file to write the mail to
@@ -82,15 +86,16 @@ class Mailer:
         '''
         # encode header values if they need to be
         charset = getattr(self.config, 'EMAIL_CHARSET', 'utf-8')
-        tracker_name = unicode(self.config.TRACKER_NAME, 'utf-8')
+        tracker_name = s2u(self.config.TRACKER_NAME)
         if not author:
             author = (tracker_name, self.config.ADMIN_EMAIL)
             name = author[0]
         else:
-            name = unicode(author[0], 'utf-8')
+            name = s2u(author[0])
         author = nice_sender_header(name, author[1], charset)
         try:
-            message['Subject'] = subject.encode('ascii')
+            subject.encode('ascii')
+            message['Subject'] = subject
         except UnicodeError:
             message['Subject'] = Header(subject, charset)
         message['To'] = ', '.join(to)
@@ -102,7 +107,8 @@ class Mailer:
 
         # Add a unique Roundup header to help filtering
         try:
-            message['X-Roundup-Name'] = tracker_name.encode('ascii')
+            tracker_name.encode('ascii')
+            message['X-Roundup-Name'] = tracker_name
         except UnicodeError:
             message['X-Roundup-Name'] = Header(tracker_name, charset)
 
@@ -111,16 +117,24 @@ class Mailer:
         # finally, an aid to debugging problems
         message['X-Roundup-Version'] = __version__
 
+    def get_text_message(self, _charset='utf-8', _subtype='plain'):
+        message = MIMENonMultipart('text', _subtype)
+        cs = Charset(_charset)
+        if cs.body_encoding == charset.BASE64:
+            cs.body_encoding = charset.QP
+        message.set_charset(cs)
+        del message['Content-Transfer-Encoding']
+        return message
+
     def get_standard_message(self, multipart=False):
         '''Form a standard email message from Roundup.
         Returns a Message object.
         '''
-        charset = getattr(self.config, 'EMAIL_CHARSET', 'utf-8')
         if multipart:
             message = MIMEMultipart()
         else:
-            message = MIMEText("")
-            message.set_charset(charset)
+            message = self.get_text_message(getattr(self.config,
+                                                    'EMAIL_CHARSET', 'utf-8'))
 
         return message
 
@@ -128,17 +142,17 @@ class Mailer:
         """Send a standard message.
 
         Arguments:
-        - to: a list of addresses usable by rfc822.parseaddr().
+        - to: a list of addresses usable by email.utils.parseaddr().
         - subject: the subject as a string.
         - content: the body of the message as a string.
         - author: the sender as a (name, address) tuple
 
         All strings are assumed to be UTF-8 encoded.
         """
+        charset = getattr(self.config, 'EMAIL_CHARSET', 'utf-8')
         message = self.get_standard_message()
         self.set_message_attributes(message, to, subject, author)
-        message.set_payload(content)
-        encode_quopri(message)
+        message.set_payload(s2u(content), charset=charset)
         self.smtp_send(to, message.as_string())
 
     def bounce_message(self, bounced_message, to, error,
@@ -146,8 +160,8 @@ class Mailer:
         """Bounce a message, attaching the failed submission.
 
         Arguments:
-        - bounced_message: an RFC822 Message object.
-        - to: a list of addresses usable by rfc822.parseaddr(). Might be
+        - bounced_message: an mailgw.RoundupMessage object.
+        - to: a list of addresses usable by email.utils.parseaddr(). Might be
           extended or overridden according to the config
           ERROR_MESSAGES_TO setting.
         - error: the reason of failure as a string.
@@ -162,7 +176,7 @@ class Mailer:
             to = None
         # see whether we should send to the dispatcher or not
         dispatcher_email = getattr(self.config, "DISPATCHER_EMAIL",
-            getattr(self.config, "ADMIN_EMAIL"))
+                                   self.config.ADMIN_EMAIL)
         error_messages_to = getattr(self.config, "ERROR_MESSAGES_TO", "user")
         if error_messages_to == "dispatcher":
             to = [dispatcher_email]
@@ -181,35 +195,27 @@ class Mailer:
         message.attach(part)
 
         # attach the original message to the returned message
-        body = []
-        for header in bounced_message.headers:
-            body.append(header)
-        try:
-            bounced_message.rewindbody()
-        except IOError, errmessage:
-            body.append("*** couldn't include message body: %s ***" %
-                errmessage)
-        else:
-            body.append('\n')
-            body.append(bounced_message.fp.read())
-        part = MIMEText(''.join(body))
+        part = MIMEText(bounced_message.flatten())
         message.attach(part)
 
+        self.logger.debug("bounce_message: to=%s, crypt_to=%s", to, crypt_to)
+
         if to:
             # send
             self.set_message_attributes(message, to, subject)
             try:
                 self.smtp_send(to, message.as_string())
-            except MessageSendError:
+            except MessageSendError as e:
                 # squash mail sending errors when bouncing mail
                 # TODO this *could* be better, as we could notify admin of the
                 # problem (even though the vast majority of bounce errors are
                 # because of spam)
+                self.logger.debug("MessageSendError: %s", str(e))
                 pass
         if crypt_to:
-            plain = pyme.core.Data(message.as_string())
-            cipher = pyme.core.Data()
-            ctx = pyme.core.Context()
+            plain = gpg.core.Data(message.as_string())
+            cipher = gpg.core.Data()
+            ctx = gpg.core.Context()
             ctx.set_armor(1)
             keys = []
             adrs = []
@@ -221,34 +227,41 @@ class Mailer:
                     adrs.append(adr)
                     keys.append(k)
                 ctx.op_keylist_end()
+            if not adrs:
+                self.logger.debug("bounce_message: no keys found for %s",
+                                  crypt_to)
             crypt_to = adrs
         if crypt_to:
             try:
                 ctx.op_encrypt(keys, 1, plain, cipher)
-                cipher.seek(0,0)
-                message=MIMEMultipart('encrypted', boundary=None,
-                    _subparts=None, protocol="application/pgp-encrypted")
-                part=MIMEBase('application', 'pgp-encrypted')
+                cipher.seek(0, 0)
+                message = MIMEMultipart('encrypted', boundary=None,
+                                        _subparts=None,
+                                        protocol="application/pgp-encrypted")
+                part = MIMEBase('application', 'pgp-encrypted')
                 part.set_payload("Version: 1\r\n")
                 message.attach(part)
-                part=MIMEBase('application', 'octet-stream')
+                part = MIMEBase('application', 'octet-stream')
                 part.set_payload(cipher.read())
                 message.attach(part)
-            except pyme.GPGMEError:
+            except gpg.GPGMEError:
+                self.logger.debug("bounce_message: Cannot encrypt to %s",
+                                  str(crypt_to))
                 crypt_to = None
         if crypt_to:
             self.set_message_attributes(message, crypt_to, subject)
             try:
                 self.smtp_send(crypt_to, message.as_string())
-            except MessageSendError:
+            except MessageSendError as e:
                 # ignore on error, see above.
+                self.logger.debug("MessageSendError: %s", str(e))
                 pass
 
     def exception_message(self):
         '''Send a message to the admins with information about the latest
         traceback.
         '''
-        subject = '%s: %s'%(self.config.TRACKER_NAME, sys.exc_info()[1])
+        subject = '%s: %s' % (self.config.TRACKER_NAME, sys.exc_info()[1])
         to = [self.config.ADMIN_EMAIL]
         content = '\n'.join(traceback.format_exception(*sys.exc_info()))
         self.standard_message(to, subject, content)
@@ -269,10 +282,12 @@ class Mailer:
             # don't send - just write to a file, use unix from line so
             # that resulting file can be openened in a mailer
             fmt = '%a %b %m %H:%M:%S %Y'
-            unixfrm = 'From %s %s' % (sender, Date ('.').pretty (fmt))
-            open(self.debug, 'a').write('%s\nFROM: %s\nTO: %s\n%s\n\n' %
-                                        (unixfrm, sender,
-                                         ', '.join(to), message))
+            unixfrm = 'From %s %s' % (sender, Date('.').pretty(fmt))
+            debug_fh = open(self.debug, 'a')
+            debug_fh.write('%s\nFROM: %s\nTO: %s\n%s\n\n' %
+                           (unixfrm, sender,
+                            ', '.join(to), message))
+            debug_fh.close()
         else:
             # now try to send the message
             try:
@@ -280,11 +295,12 @@ class Mailer:
                 # instead of to roundup
                 smtp = SMTPConnection(self.config)
                 smtp.sendmail(sender, to, message)
-            except socket.error, value:
+            except socket.error as value:
                 raise MessageSendError("Error: couldn't send email: "
-                                       "mailhost %s"%value)
-            except smtplib.SMTPException, msg:
-                raise MessageSendError("Error: couldn't send email: %s"%msg)
+                                       "mailhost %s" % value)
+            except smtplib.SMTPException as msg:
+                raise MessageSendError("Error: couldn't send email: %s" % msg)
+
 
 class SMTPConnection(smtplib.SMTP):
     ''' Open an SMTP connection to the mailhost specified in the config
@@ -297,7 +313,7 @@ class SMTPConnection(smtplib.SMTP):
         if config["MAIL_TLS"]:
             self.ehlo()
             self.starttls(config["MAIL_TLS_KEYFILE"],
-                config["MAIL_TLS_CERTFILE"])
+                          config["MAIL_TLS_CERTFILE"])
 
         # ok, now do we also need to log in?
         mailuser = config["MAIL_USERNAME"]
