@@ -1,4 +1,5 @@
-import unittest
+from __future__ import print_function
+import unittest, copy
 from cgi import FieldStorage, MiniFieldStorage
 
 from roundup import hyperdb
@@ -6,8 +7,13 @@ from roundup.date import Date, Interval
 from roundup.cgi.actions import *
 from roundup.cgi.client import add_message
 from roundup.cgi.exceptions import Redirect, Unauthorised, SeriousError, FormError
+from roundup.exceptions import Reject
 
-from mocknull import MockNull
+from roundup.anypy.cmp_ import NoneAndDictComparable
+from time import sleep
+from datetime import datetime
+
+from roundup.test.mocknull import MockNull
 
 def true(*args, **kwargs):
     return 1
@@ -16,10 +22,16 @@ class ActionTestCase(unittest.TestCase):
     def setUp(self):
         self.form = FieldStorage(environ={'QUERY_STRING': ''})
         self.client = MockNull()
+        self.client.db.Otk = MockNull()
+        self.client.db.Otk.data = {}
+        self.client.db.Otk.getall = self.data_get
+        self.client.db.Otk.set = self.data_set
+        self.client.db.config.WEB_LOGIN_ATTEMPTS_MIN = 20
+        self.client.db.config.WEB_LOGIN_EMPTY_PASSWORDS = 0
         self.client._ok_message = []
         self.client._error_message = []
-        self.client.add_error_message = lambda x : add_message(
-            self.client._error_message, x)
+        self.client.add_error_message = lambda x, escape=True: add_message(
+            self.client._error_message, x, escape=escape)
         self.client.add_ok_message = lambda x : add_message(
             self.client._ok_message, x)
         self.client.form = self.form
@@ -28,6 +40,12 @@ class ActionTestCase(unittest.TestCase):
             pass
         self.client.instance.interfaces.TemplatingUtils = TemplatingUtils
 
+    def data_get(self, key):
+        return self.client.db.Otk.data[key]
+
+    def data_set(self, key, **value):
+        self.client.db.Otk.data[key] = value
+
 class ShowActionTestCase(ActionTestCase):
     def assertRaisesMessage(self, exception, callable, message, *args,
                             **kwargs):
@@ -44,7 +62,7 @@ class ShowActionTestCase(ActionTestCase):
                 excName = exception.__name__
             else:
                 excName = str(exception)
-            raise self.failureException, excName
+            raise self.failureException(excName)
 
     def testShowAction(self):
         self.client.base = 'BASE/'
@@ -65,14 +83,38 @@ class ShowActionTestCase(ActionTestCase):
         self.assertRaisesMessage(ValueError, action.handle,
             'No type specified')
 
+    def testShowActionBadNumber(self):
+        action = ShowAction(self.client)
+        self.assertRaises(ValueError, action.handle)
+        self.form.value.append(MiniFieldStorage('@number', 'A'))
+        self.form.value.append(MiniFieldStorage('@type', 'issue'))
+        with self.assertRaises(SeriousError) as ctx:
+            action.handle()
+        self.assertEqual('"A" is not an ID (issue ID required)',
+                         ctx.exception.args[0])
+
 class RetireActionTestCase(ActionTestCase):
     def testRetireAction(self):
         self.client.db.security.hasPermission = true
         self.client._ok_message = []
         RetireAction(self.client).handle()
-        self.assert_(len(self.client._ok_message) == 1)
+        self.assertTrue(len(self.client._ok_message) == 1)
+        self.assertTrue(not self.client.nodeid == None)
+
+        self.client.template = 'index'
+        self.client._ok_message = []
+        RetireAction(self.client).handle()
+        self.assertTrue(len(self.client._ok_message) == 1)
+        self.assertTrue(self.client.nodeid == None)
+
+        self.client.env={}
+        self.client.env['REQUEST_METHOD'] = 'GET'
+        self.client._ok_message = []
+        self.assertRaises(Reject, RetireAction(self.client).execute)
+
 
     def testNoPermission(self):
+        self.client.classname='user'
         self.assertRaises(Unauthorised, RetireAction(self.client).execute)
 
     def testDontRetireAdminOrAnonymous(self):
@@ -95,7 +137,19 @@ class RestoreActionTestCase(ActionTestCase):
         self.client.db.security.hasPermission = true
         self.client._ok_message = []
         RestoreAction(self.client).handle()
-        self.assert_(len(self.client._ok_message) == 1)
+        self.assertTrue(len(self.client._ok_message) == 1)
+        self.assertTrue(not self.client.nodeid == None)
+
+        self.client.template = 'index'
+        self.client._ok_message = []
+        RestoreAction(self.client).handle()
+        self.assertTrue(len(self.client._ok_message) == 1)
+        self.assertTrue(self.client.nodeid == None)
+
+        self.client.env={}
+        self.client.env['REQUEST_METHOD'] = 'GET'
+        self.client._ok_message = []
+        self.assertRaises(Reject, RestoreAction(self.client).execute)
 
     def testNoPermission(self):
         self.assertRaises(Unauthorised, RestoreAction(self.client).execute)
@@ -152,7 +206,7 @@ class FakeFilterVarsTestCase(SearchActionTestCase):
 
     def testNumKey(self): # testing patch: http://hg.python.org/tracker/roundup/rev/98508a47c126
         for val in [ "-1000a", "test", "o0.9999", "o0", "1.00/10" ]:
-            print "testing ", val
+            print("testing ", val)
             self.client.db.classes.get_transitive_prop = lambda x: hyperdb.Number()
             self.form.value.append(MiniFieldStorage('foo', val)) # invalid numbers
             self.assertRaises(FormError, self.action.fakeFilterVars)
@@ -166,7 +220,7 @@ class FakeFilterVarsTestCase(SearchActionTestCase):
 
     def testIntKey(self): # testing patch: http://hg.python.org/tracker/roundup/rev/98508a47c126
         for val in [ "-1000a", "test", "-5E-5", "0.9999", "0.0", "1.000", "0456", "1E4" ]:
-            print "testing ", val
+            print("testing ", val)
             self.client.db.classes.get_transitive_prop = lambda x: hyperdb.Integer()
             self.form.value.append(MiniFieldStorage('foo', val))
             self.assertRaises(FormError, self.action.fakeFilterVars)
@@ -196,13 +250,22 @@ class CollisionDetectionTestCase(ActionTestCase):
         # round off for testing
         self.now.second = int(self.now.second)
 
-    def testLastUserActivity(self):
+    def testLastUserActivityAt(self):
         self.assertEqual(self.action.lastUserActivity(), None)
 
+        # test @ special variable form
         self.client.form.value.append(
             MiniFieldStorage('@lastactivity', str(self.now)))
         self.assertEqual(self.action.lastUserActivity(), self.now)
 
+    def testLastUserActivityColon(self):
+        self.assertEqual(self.action.lastUserActivity(), None)
+
+        # test : special variable form
+        self.client.form.value.append(
+            MiniFieldStorage(':lastactivity', str(self.now)))
+        self.assertEqual(self.action.lastUserActivity(), self.now)
+
     def testLastNodeActivity(self):
         self.action.classname = 'issue'
         self.action.nodeid = '1'
@@ -219,12 +282,19 @@ class CollisionDetectionTestCase(ActionTestCase):
         # fake up an actual change
         self.action.classname = 'test'
         self.action.nodeid = '1'
-        self.client.parsePropsFromForm = lambda: ({('test','1'):{1:1}}, [])
-        self.failUnless(self.action.detectCollision(self.now,
-            self.now + Interval("1d")))
-        self.failIf(self.action.detectCollision(self.now,
+        self.client.parsePropsFromForm = lambda: (
+            {('test','1'):{"prop1":"1"}}, [])
+        props = self.action.detectCollision(self.now,
+                                            self.now + Interval("1d"))
+        self.assertTrue(props)
+        self.action.handleCollision(props)
+        self.assertEqual(self.client._error_message[0],
+                         'Edit Error: someone else has edited this test '
+                         '(prop1). View <a target="_blank" href="test1">their '
+                         'changes</a> in a new window.')
+        self.assertFalse(self.action.detectCollision(self.now,
             self.now - Interval("1d")))
-        self.failIf(self.action.detectCollision(None, self.now))
+        self.assertFalse(self.action.detectCollision(None, self.now))
 
 class LoginTestCase(ActionTestCase):
     def setUp(self):
@@ -253,7 +323,7 @@ class LoginTestCase(ActionTestCase):
                 excName = exception.__name__
             else:
                 excName = str(exception)
-            raise self.failureException, excName
+            raise self.failureException(excName)
 
     def assertLoginLeavesMessages(self, messages, username=None, password=None):
         if username is not None:
@@ -302,6 +372,27 @@ class LoginTestCase(ActionTestCase):
 
         self.assertLoginLeavesMessages([], 'foo', 'right')
 
+    def testBlankPasswordLogin(self):
+        self.client.db.security.hasPermission = lambda *args, **kwargs: True
+
+        self.client.db.user.get = lambda a,b: None
+
+        def opendb(username):
+            self.assertEqual(username, 'blank')
+        self.client.opendb = opendb
+
+        self.assertEqual(self.client.db.config.WEB_LOGIN_EMPTY_PASSWORDS, 0)
+        self.assertLoginLeavesMessages(['Invalid login'], 'blank', '' )
+
+        self.client.db.config.WEB_LOGIN_EMPTY_PASSWORDS = 1
+        self.form.value[:] = []  # reset form
+        self.client._error_message = [] # reset errors
+        self.assertLoginLeavesMessages([], 'blank', '' )
+
+        # reset
+        self.client.db.user.get = lambda a,b: 'right'
+        self.client.db.config.WEB_LOGIN_EMPTY_PASSWORDS = 0
+
     def testCorrectLoginRedirect(self):
         self.client.db.security.hasPermission = lambda *args, **kwargs: True
         def opendb(username):
@@ -309,22 +400,22 @@ class LoginTestCase(ActionTestCase):
         self.client.opendb = opendb
 
         # basic test with query
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search&%40ok_message=Welcome+foo%21",
                                  'foo', 'right', "http://whoami.com/path/issue?@action=search")
 
         # test that old messages are removed
         self.form.value[:] = []         # clear out last test's setup values
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search&%40ok_message=Welcome+foo%21",
                                  'foo', 'right', "http://whoami.com/path/issue?@action=search&@ok_messagehurrah+we+win&@error_message=blam")
 
         # test when there is no query
         self.form.value[:] = []         # clear out last test's setup values
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue255",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue255?%40ok_message=Welcome+foo%21",
                                  'foo', 'right', "http://whoami.com/path/issue255")
 
         # test if we are logged out; should kill the @action=logout
         self.form.value[:] = []         # clear out last test's setup values
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue39?%40startwith=0&%40pagesize=50",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue39?%40ok_message=Welcome+foo%21&%40pagesize=50&%40startwith=0",
                                  'foo', 'right', "http://whoami.com/path/issue39?@action=logout&@pagesize=50&@startwith=0")
 
     def testInvalidLoginRedirect(self):
@@ -335,12 +426,12 @@ class LoginTestCase(ActionTestCase):
         self.client.opendb = opendb
 
         # basic test with query
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40error_message=Invalid+login&%40action=search",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search&%40error_message=Invalid+login",
                                  'foo', 'wrong', "http://whoami.com/path/issue?@action=search")
 
         # test that old messages are removed
         self.form.value[:] = []         # clear out last test's setup values
-        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40error_message=Invalid+login&%40action=search",
+        self.assertLoginRaisesRedirect("http://whoami.com/path/issue?%40action=search&%40error_message=Invalid+login",
                                  'foo', 'wrong', "http://whoami.com/path/issue?@action=search&@ok_messagehurrah+we+win&@error_message=blam")
 
         # test when there is no __came_from specified
@@ -354,6 +445,44 @@ class LoginTestCase(ActionTestCase):
         self.assertLoginRaisesRedirect("http://whoami.com/path/issue255?%40error_message=Invalid+login",
                                  'foo', 'wrong', "http://whoami.com/path/issue255")
 
+    def testLoginRateLimit(self):
+        ''' Set number of logins in setup to 20 per minute. Three second
+            delay between login attempts doesn't trip rate limit.
+            Default limit is 3/min, but that means we sleep for 20
+            seconds so I override the default limit to speed this up.
+        '''
+        # Do the first login setting an invalid login name
+        self.assertLoginLeavesMessages(['Invalid login'], 'nouser')
+        # use up the rest of the 20 login attempts. Login name
+        # persists.
+        for i in range(19):
+            self.client._error_message = []
+            self.assertLoginLeavesMessages(['Invalid login'])
+
+        self.assertRaisesMessage(Reject, LoginAction(self.client).handle,
+               'Logins occurring too fast. Please wait: 3 seconds.')
+
+        sleep(3) # sleep as requested so we can do another login
+        self.client._error_message = []
+        self.assertLoginLeavesMessages(['Invalid login']) # this is expected
+        
+        # and make sure we need to wait another three seconds
+        self.assertRaisesMessage(Reject, LoginAction(self.client).handle,
+               'Logins occurring too fast. Please wait: 3 seconds.')
+
+    def testLoginRateLimitOff(self):
+        ''' Set number of logins to 0 per minute. Verify that
+            we can do 1000 which for manual login might as well be off. 
+        '''
+
+        self.client.db.config.WEB_LOGIN_ATTEMPTS_MIN = 0
+
+        # Do the first login setting an invalid login name
+        self.assertLoginLeavesMessages(['Invalid login'], 'nouser')
+        for i in range(1000):
+            self.client._error_message = []
+            self.assertLoginLeavesMessages(['Invalid login'])
+
 class EditItemActionTestCase(ActionTestCase):
     def setUp(self):
         ActionTestCase.setUp(self)
@@ -416,7 +545,8 @@ class EditItemActionTestCase(ActionTestCase):
             self.action.handle()
         except Redirect as msg:
             pass
-        self.assertEqual(expect, self.result)
+        self.assertEqual(sorted(expect, key=NoneAndDictComparable),
+                         sorted(self.result, key=NoneAndDictComparable))
 
     def testFileAttach(self):
         expect = \
