@@ -10,6 +10,7 @@
 
 import unittest, os, shutil, errno, sys, difflib, cgi, re
 from hashlib import sha1
+from mock import Mock
 
 from roundup import init, instance, password, hyperdb, date
 
@@ -57,6 +58,7 @@ class TestDatabase:
     classes = {'test': TestClass(), 'test2': TestClass2()}
     def getUserTimezone(self):
         return 0
+    config = Mock(PASSWORD_PBKDF2_DEFAULT_ROUNDS =  1000)
 
 class RawToHyperdbTest(unittest.TestCase):
     def _test(self, propname, value, itemid=None):
@@ -88,33 +90,33 @@ class RawToHyperdbTest(unittest.TestCase):
         self.assertEqual(self._test('password', ''), None)
         self.assertEqual(self._test('password', '  a string '), 'a string')
         val = self._test('password', '  a string ')
-        self.assert_(isinstance(val, password.Password))
+        self.assertTrue(isinstance(val, password.Password))
         val = self._test('password', '{plaintext}a string')
-        self.assert_(isinstance(val, password.Password))
+        self.assertTrue(isinstance(val, password.Password))
         val = self._test('password', '{crypt}a string')
-        self.assert_(isinstance(val, password.Password))
-        s = sha1('a string').hexdigest()
+        self.assertTrue(isinstance(val, password.Password))
+        s = sha1(b'a string').hexdigest()
         val = self._test('password', '{SHA}'+s)
-        self.assert_(isinstance(val, password.Password))
+        self.assertTrue(isinstance(val, password.Password))
         self.assertEqual(val, 'a string')
         self.assertRaises(hyperdb.HyperdbValueError, self._test,
             'password', '{fubar}a string')
     def testDate(self):
         self.assertEqual(self._test('password', ''), None)
         val = self._test('date', ' 2003-01-01  ')
-        self.assert_(isinstance(val, date.Date))
+        self.assertTrue(isinstance(val, date.Date))
         val = self._test('date', ' 2003/01/01  ')
-        self.assert_(isinstance(val, date.Date))
+        self.assertTrue(isinstance(val, date.Date))
         val = self._test('date', ' 2003/1/1  ')
-        self.assert_(isinstance(val, date.Date))
+        self.assertTrue(isinstance(val, date.Date))
         val = self._test('date', ' 2003-1-1  ')
-        self.assert_(isinstance(val, date.Date))
+        self.assertTrue(isinstance(val, date.Date))
         self.assertRaises(hyperdb.HyperdbValueError, self._test, 'date',
             'fubar')
     def testInterval(self):
         self.assertEqual(self._test('password', ''), None)
         val = self._test('interval', ' +1d  ')
-        self.assert_(isinstance(val, date.Interval))
+        self.assertTrue(isinstance(val, date.Interval))
         self.assertRaises(hyperdb.HyperdbValueError, self._test, 'interval',
             'fubar')
     def testLink(self):
@@ -151,8 +153,8 @@ class RawToHyperdbTest(unittest.TestCase):
 
         with self.assertRaises(hyperdb.HyperdbValueError) as cm:
             self._test('multilink3', '1', '1')
-        self.assertEqual(cm.exception.message,
-                         "property multilink3: '1' is not a test.")
+        self.assertEqual(cm.exception.args,
+                         ("property multilink3: '1' is not a test.",))
 
         self.assertEqual(self._test('multilink3', 'valid', '1'), ['1'])
 
@@ -161,21 +163,21 @@ class RawToHyperdbTest(unittest.TestCase):
 
         with self.assertRaises(hyperdb.HyperdbValueError) as cm:
             self._test('multilink3', '+1', '1')
-        self.assertEqual(cm.exception.message,
-                         "property multilink3: '1' is not a test.")
+        self.assertEqual(cm.exception.args,
+                         ("property multilink3: '1' is not a test.",))
 
         self.assertEqual(self._test('multilink3', '+valid', '1'),
                          ['1', '2', '3'])
 
         with self.assertRaises(hyperdb.HyperdbValueError) as cm:
             self._test('multilink3', '+1,-2', '1')
-        self.assertEqual(cm.exception.message,
-                         "property multilink3: '1' is not a test.")
+        self.assertEqual(cm.exception.args,
+                         ("property multilink3: '1' is not a test.",))
 
         with self.assertRaises(hyperdb.HyperdbValueError) as cm:
             self._test('multilink3', '+valid,-2', '1')
-        self.assertEqual(cm.exception.message,
-                         "property multilink3: '2' is not a test.")
+        self.assertEqual(cm.exception.args,
+                         ("property multilink3: '2' is not a test.",))
 
         self.assertEqual(self._test('multilink3', '+valid,-2valid', '1'), ['1', '3'])
 
@@ -183,9 +185,6 @@ class RawToHyperdbTest(unittest.TestCase):
 
         self.assertEqual(self._test('multilink3', '', None), [])
 
-        with self.assertRaises(hyperdb.HyperdbValueError) as cm:
-            result = self._test('multilink3', '-valid', None)
-        self.assertEqual(cm.exception.message,
-                   "property multilink3: 'valid' is not currently an element")
+        self.assertEqual(self._test('multilink3', '-valid', None), [])
 
 # vim: set filetype=python ts=4 sw=4 et si
