@@ -1,5 +1,6 @@
-import shutil, errno, pytest, json, gzip, os, re
+import shutil, errno, pytest, json, gzip, mimetypes, os, re
 
+from roundup import date as rdate
 from roundup import i18n
 from roundup import password
 from roundup.anypy.strings import b2s
@@ -48,7 +49,9 @@ class WsgiSetup(LiveServerTestCase):
 
     dirname = '_test_instance'
     backend = 'anydbm'
-    
+
+    js_mime_type = mimetypes.guess_type("utils.js")[0]
+
     @classmethod
     def setup_class(cls):
         '''All tests in this class use the same roundup instance.
@@ -73,20 +76,30 @@ class WsgiSetup(LiveServerTestCase):
         cls.db.config.MAILHOST = "localhost"
         cls.db.config.MAIL_HOST = "localhost"
         cls.db.config.MAIL_DEBUG = "../_test_tracker_mail.log"
+
+        # added to enable csrf forgeries/CORS to be tested
         cls.db.config.WEB_CSRF_ENFORCE_HEADER_ORIGIN = "required"
         cls.db.config.WEB_ALLOWED_API_ORIGINS = "https://client.com"
+        cls.db.config['WEB_CSRF_ENFORCE_HEADER_X-REQUESTED-WITH'] = "required"
 
         # disable web login rate limiting. The fast rate of tests
         # causes them to trip the rate limit and fail.
         cls.db.config.WEB_LOGIN_ATTEMPTS_MIN = 0
         
-        cls.db.config['WEB_CSRF_ENFORCE_HEADER_X-REQUESTED-WITH'] = "required"
-
         # enable static precompressed files
         cls.db.config.WEB_USE_PRECOMPRESSED_FILES = 1
 
         cls.db.config.save()
 
+        # add an issue to allow testing retrieval.
+        # also used for text searching.
+        result = cls.db.issue.create(title="foo bar RESULT")
+
+        # add a message to allow retrieval
+        result = cls.db.msg.create(author = "1",
+                                   content = "a message foo bar RESULT",
+                                   date=rdate.Date(),
+                                   messageid="test-msg-id")
         cls.db.commit()
         cls.db.close()
         
@@ -184,13 +197,15 @@ class BaseTestCases(WsgiSetup):
         self.assertTrue(b'dauerhaft anmelden?' in f.content)
 
     def test_byte_Ranges(self):
-        """ Roundup only handles one simple two number range.
+        """ Roundup only handles one simple two number range, or
+            a single number to start from:
             Range: 10-20
-
-            The following are not supported.
-            Range: 10-20, 25-30
             Range: 10-
 
+            The following is not supported.
+            Range: 10-20, 25-30
+            Range: -10
+
             Also If-Range only supports strong etags not dates or weak etags.
 
         """
@@ -213,7 +228,27 @@ class BaseTestCases(WsgiSetup):
         self.assertEqual(f.headers['content-range'],
                          "bytes 0-10/%s"%expected_length)
 
+        # get bytes 11-21 unconditionally (0 index really??)
+        hdrs = {"Range": "bytes=10-20"}
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 206)
+        self.assertEqual(f.content, b"ge styles *")
+        # compression disabled for length < 100, so we can use 11 here
+        self.assertEqual(f.headers['content-length'], '11')
+        self.assertEqual(f.headers['content-range'],
+                         "bytes 10-20/%s"%expected_length)
+
+        # get all bytest starting from 11
+        hdrs = {"Range": "bytes=11-"}
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 206)
+        self.assertEqual(f.headers['content-range'],
+                         "bytes 11-%s/%s"%(int(expected_length) - 1,
+                                           expected_length))
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
         # conditional request 11 bytes since etag matches 206 code
+        hdrs = {"Range": "bytes=0-10"}
         hdrs['If-Range'] = etag
         f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
         self.assertEqual(f.status_code, 206)
@@ -255,7 +290,98 @@ class BaseTestCases(WsgiSetup):
         self.assertEqual(f.status_code, 416)
         self.assertEqual(f.headers['content-range'],
                          "bytes */%s"%expected_length)
-        
+
+        # invalid range multiple ranges
+        hdrs['Range'] = "bytes=0-10, 20-45"
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # invalid range is single number not number followed by -
+        hdrs['Range'] = "bytes=1"
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid first number not a number
+        hdrs['Range'] = "bytes=boom-99" # bad first value
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid last number not a number
+        hdrs['Range'] = "bytes=1-boom" # bad last value
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid first position empty
+        hdrs['Range'] = "bytes=-11" # missing first value
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid #2 < #1
+        hdrs['Range'] = "bytes=11-1" # inverted range
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid negative first number
+        hdrs['Range'] = "bytes=-1-11" # negative first number
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is invalid negative second number
+        hdrs['Range'] = "bytes=1--11" # negative second number
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+        # range is unsupported units
+        hdrs['Range'] = "badunits=1-11"
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style.css", headers=hdrs)
+        self.assertEqual(f.status_code, 200)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+        self.assertIn(b"SHA:", f.content)  # detect sha sum at end of file
+
+
+        # valid range, invalid file
+        hdrs['Range'] = "bytes=0-11"
+        print(hdrs)
+        f = requests.get(self.url_base() + "/@@file/style_nope.css",
+                         headers=hdrs)
+        self.assertEqual(f.status_code, 404)
+        self.assertNotIn('content-range', f.headers,
+                         'content-range should not be present')
+
     def test_rest_preflight_collection(self):
         # no auth for rest csrf preflight
         f = requests.options(self.url_base() + '/rest/data/user',
@@ -298,6 +424,20 @@ class BaseTestCases(WsgiSetup):
                    'allowed to use Rest Interface." } }'
         self.assertEqual(b2s(f.content), expected)
 
+        # Test when Origin is not sent.
+        f = requests.options(self.url_base() + '/rest/data/user',
+                             headers = {'content-type': "application/json",
+                             'x-requested-with': "rest",
+                             'Access-Control-Request-Headers':
+                                 "x-requested-with",
+                             'Access-Control-Request-Method': "PUT",})
+
+        self.assertEqual(f.status_code, 400)
+
+        expected = ('{ "error": { "status": 400, "msg": "Required'
+                    ' Header Missing" } }')
+        self.assertEqual(b2s(f.content), expected)
+
 
     def test_rest_invalid_method_collection(self):
         # use basic auth for rest endpoint
@@ -469,16 +609,24 @@ class BaseTestCases(WsgiSetup):
         ## test a property that doesn't exist
         f = requests.options(self.url_base() + '/rest/data/user/1/zot',
                              auth=('admin', 'sekrit'),
-                             headers = {'content-type': ""})
+                             headers = {'content-type': "",
+                                        'Origin': "http://localhost:9001",})
         print(f.status_code)
         print(f.headers)
 
         self.assertEqual(f.status_code, 404)
 
-    def test_rest_login_rate_limit(self):
+    def DISABLEtest_rest_login_rate_limit(self):
         """login rate limit applies to api endpoints. Only failure
             logins count though. So log in 10 times in a row
             to verify that valid username/passwords aren't limited.
+     
+            FIXME: client.py does not implement this. Also need a live
+            server instance that has
+
+               cls.db.config.WEB_LOGIN_ATTEMPTS_MIN = 4
+
+            not 0.
         """
 
         for i in range(10):
@@ -486,7 +634,8 @@ class BaseTestCases(WsgiSetup):
         
             f = requests.options(self.url_base() + '/rest/data',
                                  auth=('admin', 'sekrit'),
-                                 headers = {'content-type': ""}
+                                 headers = {'content-type': "",
+                                            'Origin': "http://localhost:9001",}
             )
             print(f.status_code)
             print(f.headers)
@@ -504,13 +653,17 @@ class BaseTestCases(WsgiSetup):
         
             f = requests.options(self.url_base() + '/rest/data',
                                  auth=('admin', 'ekrit'),
-                                 headers = {'content-type': ""}
+                                 headers = {'content-type': "",
+                                            'Origin': "http://localhost:9001",}
             )
             print(i, f.status_code)
             print(f.headers)
             print(f.text)
 
-            self.assertEqual(f.status_code, 401)
+            if (i < 3): # assuming limit is 4.
+                self.assertEqual(f.status_code, 401)
+            else:
+                self.assertEqual(f.status_code, 429)
 
     def test_ims(self):
         ''' retreive the user_utils.js file with old and new
@@ -526,7 +679,7 @@ class BaseTestCases(WsgiSetup):
         print(f.headers)
 
         self.assertEqual(f.status_code, 200)
-        expected = { 'Content-Type': 'application/javascript',
+        expected = { 'Content-Type': self.js_mime_type,
                      'Content-Encoding': 'gzip',
                      'Vary': 'Accept-Encoding',
         }
@@ -554,6 +707,28 @@ class BaseTestCases(WsgiSetup):
         self.assertDictEqual({ key: value for (key, value) in f.headers.items() if key in expected }, expected)
 
 
+    def test_load_issue1(self):
+        for tail in [
+                '/issue1',      # normal url
+                '/issue00001',  # leading 0's should be stripped from id
+                '/issue1>'      # surprise this works too, should it??
+        ]:
+            f = requests.get(self.url_base() + tail,
+                             headers = { 'Accept-Encoding': 'gzip',
+                                         'Accept': '*/*'})
+
+            self.assertIn(b'foo bar RESULT', f.content)
+            self.assertEqual(f.status_code, 200)
+
+    def test_load_msg1(self):
+        # leading 0's should be stripped from id
+        f = requests.get(self.url_base() + '/msg0001',
+                         headers = { 'Accept-Encoding': 'gzip',
+                                     'Accept': '*/*'})
+
+        self.assertIn(b'foo bar RESULT', f.content)
+        self.assertEqual(f.status_code, 200)
+
     def test_bad_path(self):
         f = requests.get(self.url_base() + '/_bad>',
                          headers = { 'Accept-Encoding': 'gzip, foo',
@@ -585,7 +760,7 @@ class BaseTestCases(WsgiSetup):
         print(f.headers)
 
         self.assertEqual(f.status_code, 200)
-        expected = { 'Content-Type': 'application/javascript',
+        expected = { 'Content-Type': self.js_mime_type,
                      'Content-Encoding': 'gzip',
                      'Vary': 'Accept-Encoding',
                      'Content-Length': '69',
@@ -611,7 +786,7 @@ class BaseTestCases(WsgiSetup):
         print(f.headers)
 
         self.assertEqual(f.status_code, 200)
-        expected = { 'Content-Type': 'application/javascript',
+        expected = { 'Content-Type': self.js_mime_type,
                      'Content-Encoding': 'br',
                      'Vary': 'Accept-Encoding',
                      'Content-Length': '960',
@@ -645,7 +820,7 @@ class BaseTestCases(WsgiSetup):
         print(f.headers)
 
         self.assertEqual(f.status_code, 200)
-        expected = { 'Content-Type': 'application/javascript',
+        expected = { 'Content-Type': self.js_mime_type,
                      'Content-Encoding': 'gzip',
                      'Vary': 'Accept-Encoding',
         }
@@ -788,7 +963,7 @@ class BaseTestCases(WsgiSetup):
                              headers = {'content-type': "",
                                         'Accept-Encoding': '%s, foo'%method,
                                         'Accept': '*/*',
-                                        'Origin': 'ZZZZ'})
+                                        'Origin': 'https://client.com'})
         print(f.status_code)
         print(f.headers)
 
@@ -796,7 +971,7 @@ class BaseTestCases(WsgiSetup):
         self.assertEqual(f.status_code, 400)
         expected = { 'Content-Type': 'application/json',
                      'Access-Control-Allow-Credentials': 'true',
-                     'Access-Control-Allow-Origin': 'ZZZZ',
+                     'Access-Control-Allow-Origin': 'https://client.com',
                      'Allow': 'OPTIONS, GET, POST, PUT, DELETE, PATCH',
                      'Vary': 'Origin'
         }
@@ -823,7 +998,7 @@ class BaseTestCases(WsgiSetup):
         print(f.headers)
 
         self.assertEqual(f.status_code, 200)
-        expected = { 'Content-Type': 'application/javascript',
+        expected = { 'Content-Type': self.js_mime_type,
                      'Content-Encoding': method,
                      'Vary': 'Accept-Encoding',
         }
@@ -903,6 +1078,36 @@ class BaseTestCases(WsgiSetup):
         self.assertEqual(f.status_code, 200)
         self.assertEqual(f.headers['Cache-Control'], 'public, max-age=1209600')
 
+    def test_missing_session_key(self):
+        '''Test case where we have an outdated session cookie. Make
+           sure cookie is removed.
+        '''
+        session = requests.Session()
+        session.headers.update({'Origin': 'http://localhost:9001'})
+
+        # login using form to get cookie
+        login = {"__login_name": 'admin', '__login_password': 'sekrit',
+                 "@action": "login"}
+        f = session.post(self.url_base()+'/', data=login)
+        
+        # verify cookie is present and we are logged in
+        self.assertIn('<b>Hello, admin</b>', f.text)
+        self.assertIn('roundup_session_Roundupissuetracker',
+                         session.cookies)
+
+        f = session.get(self.url_base()+'/')
+        self.assertIn('<b>Hello, admin</b>', f.text)
+
+        for cookie in session.cookies:
+            if cookie.name == 'roundup_session_Roundupissuetracker':
+                cookie.value = 'bad_cookie_no_chocolate'
+                break
+
+        f = session.get(self.url_base()+'/')
+
+        self.assertNotIn('<b>Hello, admin</b>', f.text)
+        self.assertNotIn('roundup_session_Roundupissuetracker', session.cookies)
+
     def test_login_fail_then_succeed(self):
         # Set up session to manage cookies <insert blue monster here>
         session = requests.Session()
@@ -992,6 +1197,7 @@ class BaseTestCases(WsgiSetup):
         # download file and verify content
         f = session.get(self.url_base()+'/file%(file)s/text1.txt'%m.groupdict())
         self.assertEqual(f.text, file_content)
+        self.assertEqual(f.headers["X-Content-Type-Options"], "nosniff")
         print(f.text)
 
     def test_new_file_via_rest(self):
@@ -1050,6 +1256,10 @@ class BaseTestCases(WsgiSetup):
         )
         self.assertEqual(r.status_code, 201)
         print(r.status_code)
+        
+    def test_fts(self):
+        f = requests.get(self.url_base() + "?@search_text=RESULT")
+        self.assertIn("foo bar", f.text)
 
 class TestFeatureFlagCacheTrackerOn(BaseTestCases, WsgiSetup):
     """Class to run all test in BaseTestCases with the cache_tracker
@@ -1064,3 +1274,90 @@ class TestFeatureFlagCacheTrackerOn(BaseTestCases, WsgiSetup):
             # wsgiref/validator.py InputWrapper::readline is broke and
             # doesn't support the max bytes to read argument.
             return RequestDispatcher(self.dirname, feature_flags=ff)
+
+class TestPostgresWsgiServer(BaseTestCases, WsgiSetup):
+    """Class to run all test in BaseTestCases with the cache_tracker
+       feature flag enabled when starting the wsgi server
+    """
+
+    backend = 'postgresql'
+
+    @classmethod
+    def setup_class(cls):
+        '''All tests in this class use the same roundup instance.
+           This instance persists across all tests.
+           Create the tracker dir here so that it is ready for the
+           create_app() method to be called.
+
+           cribbed from WsgiSetup::setup_class
+        '''
+
+        # tests in this class.
+        # set up and open a tracker
+        cls.instance = db_test_base.setupTracker(cls.dirname, cls.backend)
+
+        # open the database
+        cls.db = cls.instance.open('admin')
+
+        # add a user without edit access for status.
+        cls.db.user.create(username="fred", roles='User',
+            password=password.Password('sekrit'), address='fred@example.com')
+
+        # set the url the test instance will run at.
+        cls.db.config['TRACKER_WEB'] = "http://localhost:9001/"
+        # set up mailhost so errors get reported to debuging capture file
+        cls.db.config.MAILHOST = "localhost"
+        cls.db.config.MAIL_HOST = "localhost"
+        cls.db.config.MAIL_DEBUG = "../_test_tracker_mail.log"
+
+        # added to enable csrf forgeries/CORS to be tested
+        cls.db.config.WEB_CSRF_ENFORCE_HEADER_ORIGIN = "required"
+        cls.db.config.WEB_ALLOWED_API_ORIGINS = "https://client.com"
+        cls.db.config['WEB_CSRF_ENFORCE_HEADER_X-REQUESTED-WITH'] = "required"
+
+        cls.db.config.INDEXER = "native-fts"
+
+        # disable web login rate limiting. The fast rate of tests
+        # causes them to trip the rate limit and fail.
+        cls.db.config.WEB_LOGIN_ATTEMPTS_MIN = 0
+        
+        # enable static precompressed files
+        cls.db.config.WEB_USE_PRECOMPRESSED_FILES = 1
+
+        cls.db.config.save()
+
+        cls.db.commit()
+        cls.db.close()
+
+        # re-open the database to get the updated INDEXER
+        cls.db = cls.instance.open('admin')
+
+        result = cls.db.issue.create(title="foo bar RESULT")
+
+        # add a message to allow retrieval
+        result = cls.db.msg.create(author = "1",
+                                   content = "a message foo bar RESULT",
+                                   date=rdate.Date(),
+                                   messageid="test-msg-id")
+
+        cls.db.commit()
+        cls.db.close()
+        
+        # Force locale config to find locales in checkout not in
+        # installed directories
+        cls.backup_domain = i18n.DOMAIN
+        cls.backup_locale_dirs = i18n.LOCALE_DIRS
+        i18n.LOCALE_DIRS = ['locale']
+        i18n.DOMAIN = ''
+
+    @classmethod
+    def tearDownClass(cls):
+        # cleanup
+        cls.instance.backend.db_nuke(cls.db.config)
+
+    def test_native_fts(self):
+        self.assertIn("postgresql_fts", str(self.db.indexer))
+
+        # use a ts: search as well so it only works on postgres_fts indexer
+        f = requests.get(self.url_base() + "?@search_text=ts:RESULT")
+        self.assertIn("foo bar RESULT", f.text)
