@@ -15,12 +15,30 @@
 # BASIS, AND THERE IS NO OBLIGATION WHATSOEVER TO PROVIDE MAINTENANCE,
 # SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 
+import email
 import unittest
-from cStringIO import StringIO
+from roundup.anypy.strings import StringIO
 
-from roundup.mailgw import Message
+from roundup.mailgw import RoundupMessage
+
+def gen_message(spec):
+    """Create a basic MIME message according to 'spec'.
+
+    Each line of a spec has one content-type, which is optionally indented.
+    The indentation signifies how deep in the MIME hierarchy the
+    content-type is.
+
+    """
+
+    def getIndent(line):
+        """Get the current line's indentation, using four-space indents."""
+        count = 0
+        for char in line:
+            if char != ' ':
+                break
+            count += 1
+        return count // 4
 
-class ExampleMessage(Message):
     # A note on message/rfc822: The content of such an attachment is an
     # email with at least one header line. RFC2046 tells us: """   A
     # media type of "message/rfc822" indicates that the body contains an
@@ -42,36 +60,22 @@ class ExampleMessage(Message):
              'application/pdf': '    name="foo.pdf"\nfoo\n',
              'message/rfc822': '\nSubject: foo\n\nfoo\n'}
 
-    def __init__(self, spec):
-        """Create a basic MIME message according to 'spec'.
+    parts = []
+    for line in spec.splitlines():
+        content_type = line.strip()
+        if not content_type:
+            continue
 
-        Each line of a spec has one content-type, which is optionally indented.
-        The indentation signifies how deep in the MIME hierarchy the
-        content-type is.
+        indent = getIndent(line)
+        if indent:
+            parts.append('\n--boundary-%s\n' % indent)
+        parts.append('Content-type: %s;\n' % content_type)
+        parts.append(table[content_type] % {'indent': indent + 1})
 
-        """
-        parts = []
-        for line in spec.splitlines():
-            content_type = line.strip()
-            if not content_type:
-                continue
+    for i in range(indent, 0, -1):
+        parts.append('\n--boundary-%s--\n' % i)
 
-            indent = self.getIndent(line)
-            if indent:
-                parts.append('\n--boundary-%s\n' % indent)
-            parts.append('Content-type: %s;\n' % content_type)
-            parts.append(self.table[content_type] % {'indent': indent + 1})
-
-        Message.__init__(self, StringIO(''.join(parts)))
-
-    def getIndent(self, line):
-        """Get the current line's indentation, using four-space indents."""
-        count = 0
-        for char in line:
-            if char != ' ':
-                break
-            count += 1
-        return count / 4
+    return email.message_from_file(StringIO(''.join(parts)), RoundupMessage)
 
 class MultipartTestCase(unittest.TestCase):
     def setUp(self):
@@ -110,55 +114,50 @@ class MultipartTestCase(unittest.TestCase):
         self.fp.seek(0)
 
     def testMultipart(self):
-        m = Message(self.fp)
-        self.assert_(m is not None)
+        m = email.message_from_file(self.fp, RoundupMessage)
+        self.assertTrue(m is not None)
 
-        # skip the first bit
-        p = m.getpart()
-        self.assert_(p is not None)
-        self.assertEqual(p.fp.read(),
-            'This is a multipart message. Ignore this bit.\r\n')
+        it = iter(m.get_payload())
 
         # first text/plain
-        p = m.getpart()
-        self.assert_(p is not None)
-        self.assertEqual(p.gettype(), 'text/plain')
-        self.assertEqual(p.fp.read(),
+        p = next(it, None)
+        self.assertTrue(p is not None)
+        self.assertEqual(p.get_content_type(), 'text/plain')
+        self.assertEqual(p.get_payload(),
             'Hello, world!\r\n\r\nBlah blah\r\nfoo\r\n-foo\r\n')
 
         # sub-multipart
-        p = m.getpart()
-        self.assert_(p is not None)
-        self.assertEqual(p.gettype(), 'multipart/alternative')
+        p = next(it, None)
+        self.assertTrue(p is not None)
+        self.assertEqual(p.get_content_type(), 'multipart/alternative')
 
         # sub-multipart text/plain
-        q = p.getpart()
-        self.assert_(q is not None)
-        q = p.getpart()
-        self.assert_(q is not None)
-        self.assertEqual(q.gettype(), 'text/plain')
-        self.assertEqual(q.fp.read(), 'Hello, world!\r\n\r\nBlah blah\r\n')
+        qit = iter(p.get_payload())
+        q = next(qit, None)
+        self.assertTrue(q is not None)
+        self.assertEqual(q.get_content_type(), 'text/plain')
+        self.assertEqual(q.get_payload(), 'Hello, world!\r\n\r\nBlah blah\r\n')
 
         # sub-multipart text/html
-        q = p.getpart()
-        self.assert_(q is not None)
-        self.assertEqual(q.gettype(), 'text/html')
-        self.assertEqual(q.fp.read(), '<b>Hello, world!</b>\r\n')
+        q = next(qit, None)
+        self.assertTrue(q is not None)
+        self.assertEqual(q.get_content_type(), 'text/html')
+        self.assertEqual(q.get_payload(), '<b>Hello, world!</b>\r\n')
 
         # sub-multipart end
-        q = p.getpart()
-        self.assert_(q is None)
+        q = next(qit, None)
+        self.assertTrue(q is None)
 
         # final text/plain
-        p = m.getpart()
-        self.assert_(p is not None)
-        self.assertEqual(p.gettype(), 'text/plain')
-        self.assertEqual(p.fp.read(),
+        p = next(it, None)
+        self.assertTrue(p is not None)
+        self.assertEqual(p.get_content_type(), 'text/plain')
+        self.assertEqual(p.get_payload(),
             'Last bit\n')
 
         # end
-        p = m.getpart()
-        self.assert_(p is None)
+        p = next(it, None)
+        self.assertTrue(p is None)
 
     def TestExtraction(self, spec, expected, convert_html_with=False):
         if convert_html_with:
@@ -167,7 +166,7 @@ class MultipartTestCase(unittest.TestCase):
         else:
             html2text=None
 
-        self.assertEqual(ExampleMessage(spec).extract_content(
+        self.assertEqual(gen_message(spec).extract_content(
             html2text=html2text), expected)
 
     def testTextPlain(self):
@@ -187,7 +186,7 @@ multipart/mixed
     text/plain
     application/pdf""",
                   ('foo\n',
-                   [('foo.pdf', 'application/pdf', 'foo\n')], False))
+                   [('foo.pdf', 'application/pdf', b'foo\n')], False))
 
     def testMultipartMixedHtml(self):
         # test with html conversion enabled
@@ -198,7 +197,7 @@ multipart/mixed
                   ('bar >\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                   ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                   ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with='dehtml')
 
         # test with html conversion disabled
@@ -209,7 +208,7 @@ multipart/mixed
                   (None,
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with=False)
 
     def testMultipartAlternative(self):
@@ -217,7 +216,7 @@ multipart/mixed
 multipart/alternative
     text/plain
     application/pdf
-        """, ('foo\n', [('foo.pdf', 'application/pdf', 'foo\n')], False))
+        """, ('foo\n', [('foo.pdf', 'application/pdf', b'foo\n')], False))
 
     def testMultipartAlternativeHtml(self):
         self.TestExtraction("""
@@ -227,7 +226,7 @@ multipart/alternative
                   ('bar >\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                   ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                   ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with='dehtml')
 
         self.TestExtraction("""
@@ -237,7 +236,7 @@ multipart/alternative
                   (None,
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with=False)
 
     def testMultipartAlternativeHtmlText(self):
@@ -250,7 +249,7 @@ multipart/alternative
                   ('foo\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with='dehtml')
 
         # text should take priority over html when text is first
@@ -262,7 +261,7 @@ multipart/alternative
                   ('foo\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with='dehtml')
 
         # text should take priority over html when text is second and
@@ -275,7 +274,7 @@ multipart/alternative
                   ('foo\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with=False)
 
         # text should take priority over html when text is first and
@@ -288,7 +287,7 @@ multipart/alternative
                   ('foo\n',
                    [('bar.html', 'text/html',
                       '<html><body>bar &gt;</body></html>\n'),
-                    ('foo.pdf', 'application/pdf', 'foo\n')], False),
+                    ('foo.pdf', 'application/pdf', b'foo\n')], False),
                             convert_html_with=False)
 
     def testDeepMultipartAlternative(self):
@@ -297,7 +296,7 @@ multipart/mixed
     multipart/alternative
         text/plain
         application/pdf
-        """, ('foo\n', [('foo.pdf', 'application/pdf', 'foo\n')], False))
+        """, ('foo\n', [('foo.pdf', 'application/pdf', b'foo\n')], False))
 
     def testSignedText(self):
         self.TestExtraction("""
@@ -313,7 +312,7 @@ multipart/signed
         application/pdf
     application/pgp-signature""",
                   ('foo\n',
-                   [('foo.pdf', 'application/pdf', 'foo\n')], False))
+                   [('foo.pdf', 'application/pdf', b'foo\n')], False))
 
     def testAttachedSignature(self):
         self.TestExtraction("""
@@ -321,7 +320,7 @@ multipart/mixed
     text/plain
     application/pgp-signature""",
                   ('foo\n',
-                   [('foo.gpg', 'application/pgp-signature', 'foo\n')], False))
+                   [('foo.gpg', 'application/pgp-signature', b'foo\n')], False))
 
     def testMessageRfc822(self):
         self.TestExtraction("""
