@@ -19,7 +19,9 @@
 # SOFTWARE.
 
 from __future__ import print_function
-import os, unittest, shutil
+import os
+import shutil
+import unittest
 
 from roundup import backends
 import roundup.password
@@ -41,19 +43,23 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         # TODO: some asserts
 
     def testInitialiseSecurity(self):
-        ei = self.db.security.addPermission(name="Edit", klass="issue",
-                        description="User is allowed to edit issues")
+        ei = self.db.security.addPermission(
+            name="Edit", klass="issue",
+            description="User is allowed to edit issues")
         self.db.security.addPermissionToRole('User', ei)
-        ai = self.db.security.addPermission(name="View", klass="issue",
-                        description="User is allowed to access issues")
+        ai = self.db.security.addPermission(
+            name="View", klass="issue",
+            description="User is allowed to access issues")
         self.db.security.addPermissionToRole('User', ai)
 
     def testAdmin(self):
-        ei = self.db.security.addPermission(name="Edit", klass="issue",
-                        description="User is allowed to edit issues")
+        ei = self.db.security.addPermission(
+            name="Edit", klass="issue",
+            description="User is allowed to edit issues")
         self.db.security.addPermissionToRole('User', ei)
-        ei = self.db.security.addPermission(name="Edit", klass=None,
-                        description="User is allowed to edit issues")
+        ei = self.db.security.addPermission(
+            name="Edit", klass=None,
+            description="User is allowed to edit issues")
         self.db.security.addPermissionToRole('Admin', ei)
 
         u1 = self.db.user.create(username='one', roles='Admin')
@@ -62,13 +68,12 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         self.assertTrue(self.db.security.hasPermission('Edit', u1, None))
         self.assertTrue(not self.db.security.hasPermission('Edit', u2, None))
 
-
     def testGetPermission(self):
         self.db.security.getPermission('Edit')
         self.db.security.getPermission('View')
         self.assertRaises(ValueError, self.db.security.getPermission, 'x')
         self.assertRaises(ValueError, self.db.security.getPermission, 'Edit',
-            'fubar')
+                          'fubar')
 
         add = self.db.security.addPermission
         get = self.db.security.getPermission
@@ -83,9 +88,11 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         epi1 = add(name="Edit", klass="issue", properties=['title'])
         self.assertEqual(get('Edit', 'issue', properties=['title']), epi1)
         epi2 = add(name="Edit", klass="issue", properties=['title'],
-                  props_only=True)
-        self.assertEqual(get('Edit', 'issue', properties=['title'], props_only=False), epi1)
-        self.assertEqual(get('Edit', 'issue', properties=['title'], props_only=True), epi2)
+                   props_only=True)
+        self.assertEqual(get('Edit', 'issue', properties=['title'],
+                             props_only=False), epi1)
+        self.assertEqual(get('Edit', 'issue', properties=['title'],
+                             props_only=True), epi2)
         self.db.security.set_props_only_default(True)
         self.assertEqual(get('Edit', 'issue', properties=['title']), epi2)
         api1 = add(name="View", klass="issue", properties=['title'])
@@ -94,7 +101,7 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         api2 = add(name="View", klass="issue", properties=['title'])
         self.assertEqual(get('View', 'issue', properties=['title']), api2)
         self.assertNotEqual(get('View', 'issue', properties=['title']), api1)
-        
+
         # check function
         dummy = lambda: 0
         eci = add(name="Edit", klass="issue", check=dummy)
@@ -102,36 +109,36 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         # props_only only makes sense if you are setting props.
         # make it a no-op unless properties is set.
         self.assertEqual(get('Edit', 'issue', check=dummy,
-                              props_only=True), eci)
+                             props_only=True), eci)
         aci = add(name="View", klass="issue", check=dummy)
         self.assertEqual(get('View', 'issue', check=dummy), aci)
 
         # all
         epci = add(name="Edit", klass="issue", properties=['title'],
-            check=dummy)
+                   check=dummy)
 
         self.db.security.set_props_only_default(False)
         # implicit props_only=False
         self.assertEqual(get('Edit', 'issue', properties=['title'],
-                              check=dummy), epci)
+                             check=dummy), epci)
         # explicit props_only=False
         self.assertEqual(get('Edit', 'issue', properties=['title'],
-                              check=dummy, props_only=False), epci)
+                             check=dummy, props_only=False), epci)
 
         # implicit props_only=True
         self.db.security.set_props_only_default(True)
         self.assertRaises(ValueError, get, 'Edit', 'issue',
-                                          properties=['title'],
-                                          check=dummy)
+                          properties=['title'],
+                          check=dummy)
         # explicit props_only=False
         self.assertRaises(ValueError, get, 'Edit', 'issue',
-                                          properties=['title'],
-                                          check=dummy, props_only=True)
+                          properties=['title'],
+                          check=dummy, props_only=True)
 
         apci = add(name="View", klass="issue", properties=['title'],
-            check=dummy)
+                   check=dummy)
         self.assertEqual(get('View', 'issue', properties=['title'],
-            check=dummy), apci)
+                             check=dummy), apci)
 
         # Reset to default. Somehow this setting looks like it
         # was bleeding through to other tests in test_xmlrpc.
@@ -165,18 +172,19 @@ class PermissionTest(MyTestCase, unittest.TestCase):
 
         # property
         addRole(name='Role2')
-        addToRole('Role2', add(name="Test", klass="test", properties=['a','b']))
+        addToRole('Role2', add(name="Test", klass="test",
+                               properties=['a', 'b']))
         user2 = self.db.user.create(username='user2', roles='Role2')
 
         # check function
         check_old_style = lambda db, userid, itemid: itemid == '2'
-        #def check_old_style(db, userid, itemid):
+        # def check_old_style(db, userid, itemid):
         #    print "checking userid, itemid: %r"%((userid,itemid),)
         #    return(itemid == '2')
 
         # setup to check function new style. Make sure that
         # other args are passed.
-        def check(db,userid,itemid, **other):
+        def check(db, userid, itemid, **other):
             prop = other['property']
             prop = other['classname']
             prop = other['permission']
@@ -185,7 +193,7 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         # also create a check as a callable of a class
         #   https://issues.roundup-tracker.org/issue2550952
         class CheckClass(object):
-            def __call__(self, db,userid,itemid, **other):
+            def __call__(self, db, userid, itemid, **other):
                 prop = other['property']
                 prop = other['classname']
                 prop = other['permission']
@@ -240,7 +248,6 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         self.assertEqual(has('Test', user7, 'test'), 1)
         self.assertEqual(has('Test', none, 'test'), 0)
 
-
         # *any* access to item
         self.assertEqual(has('Test', user1, 'test', itemid='1'), 1)
         self.assertEqual(has('Test', user2, 'test', itemid='1'), 1)
@@ -313,48 +320,48 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         # now mix property and check commands
         # check is old style props_only = false
         self.assertEqual(has('Test', user7, 'test', property="c",
-                              itemid='2'), 0)
+                             itemid='2'), 0)
         self.assertEqual(has('Test', user7, 'test', property="c",
-                              itemid='1'), 0)
+                             itemid='1'), 0)
 
         self.assertEqual(has('Test', user7, 'test', property="a",
-                              itemid='2'), 1)
+                             itemid='2'), 1)
         self.assertEqual(has('Test', user7, 'test', property="a",
-                              itemid='1'), 0)
+                             itemid='1'), 0)
 
         # check is new style props_only = false
         self.assertEqual(has('Test', user6, 'test', itemid='2',
-                              property='c'), 0)
+                             property='c'), 0)
         self.assertEqual(has('Test', user6, 'test', itemid='1',
-                              property='c'), 0)
+                             property='c'), 0)
         self.assertEqual(has('Test', user6, 'test', itemid='2',
-                              property='b'), 0)
+                             property='b'), 0)
         self.assertEqual(has('Test', user6, 'test', itemid='1',
-                              property='b'), 1)
+                             property='b'), 1)
         self.assertEqual(has('Test', user6, 'test', itemid='2',
-                              property='a'), 0)
+                             property='a'), 0)
         self.assertEqual(has('Test', user6, 'test', itemid='1',
-                              property='a'), 1)
+                             property='a'), 1)
 
         # check is old style props_only = true
         self.assertEqual(has('Test', user5, 'test', itemid='2',
-                              property='b'), 0)
+                             property='b'), 0)
         self.assertEqual(has('Test', user5, 'test', itemid='1',
-                              property='b'), 0)
+                             property='b'), 0)
         self.assertEqual(has('Test', user5, 'test', itemid='2',
-                              property='a'), 1)
+                             property='a'), 1)
         self.assertEqual(has('Test', user5, 'test', itemid='1',
-                              property='a'), 0)
+                             property='a'), 0)
 
         # check is new style props_only = true
         self.assertEqual(has('Test', user4, 'test', itemid='2',
-                              property='b'), 0)
+                             property='b'), 0)
         self.assertEqual(has('Test', user4, 'test', itemid='1',
-                              property='b'), 0)
+                             property='b'), 0)
         self.assertEqual(has('Test', user4, 'test', itemid='2',
-                              property='a'), 0)
+                             property='a'), 0)
         self.assertEqual(has('Test', user4, 'test', itemid='1',
-                              property='a'), 1)
+                             property='a'), 1)
 
     def testTransitiveSearchPermissions(self):
         add = self.db.security.addPermission
@@ -420,6 +427,173 @@ class PermissionTest(MyTestCase, unittest.TestCase):
         roundup.password.crypt = None
         with self.assertRaises(roundup.password.PasswordValueError) as ctx:
             roundup.password.test_missing_crypt()
+        self.assertEqual(ctx.exception.args[0],
+                         "Unsupported encryption scheme 'crypt'")
         roundup.password.crypt = orig_crypt
 
+    def test_pbkdf2_unpack_errors(self):
+        pbkdf2_unpack = roundup.password.pbkdf2_unpack
+
+        with self.assertRaises(roundup.password.PasswordValueError) as ctx:
+            pbkdf2_unpack("fred$password")
+
+        self.assertEqual(ctx.exception.args[0],
+                         'invalid PBKDF2 hash (wrong number of separators)')
+
+        with self.assertRaises(roundup.password.PasswordValueError) as ctx:
+            pbkdf2_unpack("0200000$salt$password")
+
+        self.assertEqual(ctx.exception.args[0],
+                         'invalid PBKDF2 hash (zero-padded rounds)')
+
+        with self.assertRaises(roundup.password.PasswordValueError) as ctx:
+            pbkdf2_unpack("fred$salt$password")
+
+        self.assertEqual(ctx.exception.args[0],
+                         'invalid PBKDF2 hash (invalid rounds)')
+
+    def test_empty_passwords(self):
+
+        p = roundup.password.Password()
+
+        with self.assertRaises(ValueError) as ctx:
+            p == "foo"
+
+        self.assertEqual(ctx.exception.args[0],
+                         'Password not set')
+
+        with self.assertRaises(ValueError) as ctx:
+            p.__str__()
+
+        self.assertEqual(ctx.exception.args[0],
+                         'Password not set')
+
+        # make sure it uses the default scheme
+        default_scheme = roundup.password.Password.default_scheme
+        p.setPassword("sekret", config=self.db.config)
+        self.assertEqual(p.scheme, default_scheme)
+
+    def test_pbkdf2_migrate_rounds(self):
+        '''Check that migration happens when number of rounds in
+           config is larger than number of rounds in current password.
+        '''
+
+        p = roundup.password.Password('sekrit', 'PBKDF2',
+                                      config=self.db.config)
+
+        self.db.config.PASSWORD_PBKDF2_DEFAULT_ROUNDS = 2000000
+
+        os.environ["PYTEST_USE_CONFIG"] = "True"
+        self.assertEqual(p.needs_migration(config=self.db.config), True)
+        del(os.environ["PYTEST_USE_CONFIG"])
+
+        # set up p with rounds under 1000. This is usually prevented,
+        # but older software could generate smaller rounds.
+        p.password = p.password.replace('1000$', '900$')
+        self.assertEqual(p.needs_migration(config=self.db.config), True)
+
+    def test_encodePassword_errors(self):
+        self.db.config.PASSWORD_PBKDF2_DEFAULT_ROUNDS = 999
+
+        os.environ["PYTEST_USE_CONFIG"] = "True"
+        with self.assertRaises(roundup.password.PasswordValueError) as ctx:
+            roundup.password.encodePassword('sekrit', 'PBKDF2',
+                                            config=self.db.config)
+
+        self.assertEqual(ctx.exception.args[0],
+                         'invalid PBKDF2 hash (rounds too low)')
+
+        del(os.environ["PYTEST_USE_CONFIG"])
+
+        with self.assertRaises(roundup.password.PasswordValueError) as ctx:
+            roundup.password.encodePassword('sekrit', 'fred',
+                                            config=self.db.config)
+
+        self.assertEqual(ctx.exception.args[0],
+                         "Unknown encryption scheme 'fred'")
+
+    def test_pbkdf2_errors(self):
+
+        with self.assertRaises(ValueError) as ctx:
+            roundup.password.pbkdf2('sekret', b'saltandpepper', 0, 41)
+
+        self.assertEqual(ctx.exception.args[0],
+                         "key length too large")
+
+        with self.assertRaises(ValueError) as ctx:
+            roundup.password.pbkdf2('sekret', b'saltandpepper', 0, 40)
+
+        self.assertEqual(ctx.exception.args[0],
+                         "rounds must be positive number")
+
+    def test_pbkdf2_sha512_errors(self):
+
+        with self.assertRaises(ValueError) as ctx:
+            roundup.password.pbkdf2_sha512('sekret', b'saltandpepper', 0, 65)
+
+        self.assertEqual(ctx.exception.args[0],
+                         "key length too large")
+
+        with self.assertRaises(ValueError) as ctx:
+            roundup.password.pbkdf2_sha512('sekret', b'saltandpepper', 0, 64)
+
+        self.assertEqual(ctx.exception.args[0],
+                         "rounds must be positive number")
+
+    def test_misc_functions(self):
+        import random  # for fuzzing later
+
+        v = roundup.password.bchr(64)
+        if bytes == str:
+            self.assertEqual(v, '@')
+        else:
+            self.assertEqual(v, b'@')
+
+        v = roundup.password.bord(b'@')
+        if bytes == str:
+            self.assertEqual(v, 64)
+        else:
+            self.assertEqual(v, b'@')
+
+        for plain, encode in (
+                (b'tes', 'dGVz'),
+                (b'test', 'dGVzdA'),
+                (b'testb', "dGVzdGI"),
+        ):
+            v = roundup.password.h64encode(plain)
+            self.assertEqual(v, encode)
+            v = roundup.password.h64decode(v)
+            self.assertEqual(v, plain)
+
+        with self.assertRaises(ValueError) as ctx:
+            v = roundup.password.h64decode("dGVzd")
+            self.assertEqual(ctx.exception.args[0], "Invalid base64 input")
+
+        # poor man's fuzzer
+        if bytes == str:
+            # alias range to xrange for python2, more efficient.
+            range_ = xrange  # noqa: F821
+        else:
+            range_ = range
+
+        for i in range_(25):
+            plain = bytearray(random.getrandbits(8) for _ in range_(i*4))
+            e = roundup.password.h64encode(plain)
+            self.assertEqual(roundup.password.h64decode(e), plain)
+
+    def test_encodePasswordNoConfig(self):
+        # should run cleanly as we are in a test.
+        #
+        p = roundup.password.encodePassword('sekrit', 'PBKDF2')
+        # verify 1000 rounds being used becaue we are in test mode
+        self.assertTrue(p.startswith("1000$"))
+
+        del(os.environ["PYTEST_CURRENT_TEST"])
+        self.assertNotIn("PYTEST_CURRENT_TEST", os.environ)
+
+        with self.assertRaises(roundup.password.ConfigNotSet) as ctx:
+            roundup.password.encodePassword('sekrit', 'PBKDF2')
+
+        self.assertEqual(ctx.exception.args[0],
+                         "encodePassword called without config.")
 # vim: set filetype=python sts=4 sw=4 et si :
